@@ -1,8 +1,8 @@
 """
-__init__.py
-
 High-level evaluation tools made available with just `import potluck`.
 
+__init__.py
+
 Typically, one would call `load_configuration`, then `setup`, and finally
 one of the `launch_*` functions.
 
@@ -10,6 +10,7 @@ Dependencies:
 
 - `jinja2` for report rendering via HTML templates.
 - `pygments` for code highlighting in reports.
+- `importlib_resources` for resource loading.
 
 This module relies on a configuration file to find task meta-data, task
 specifications, and the submission it will evaluate. Call
@@ -25,7 +26,6 @@ is the place to get started, while `potluck.rubrics` and
 far into the weeds. There are however many sub-modules that take care of
 the complicated evaluation process:
 
-- `potluck.analyze`: Analyzes submitted code.
 - `potluck.compare`: Compares values and describes differences.
 - `potluck.context_utils`: Utilities used by contexts and also elsewhere.
 - `potluck.contexts`: Caching and dependencies for test results.
@@ -40,6 +40,7 @@ the complicated evaluation process:
 - `potluck.logging`: Manages logging of the evaluation process.
 - `potluck.mast`: AST matching, courtesy Ben Wood.
 - `potluck.mast_utils`: Utilities used for AST matching.
+- `potluck.meta`: Meta-testing for checking your specifications.
 - `potluck.patterns`: Common patterns for AST matching.
 - `potluck.phrasing`: Low-level language functions for generating
   English feedback (e.g., pluralization).
@@ -59,12 +60,11 @@ from . import load
 from . import rubrics
 from . import report
 from . import contexts
+from . import meta
 
 
-__version__ = "0.5"
-"""
-Package version variable.
-"""
+# Import version
+from ._version import __version__
 
 
 def load_configuration(config_module_name):
@@ -114,7 +114,7 @@ def generate_rubric(task_info, rubric_filename):
     )
 
     # Ensure we've got a place to put our report
-    os.makedirs(os.path.split(rubric_filename)[0], exist_ok=True)
+    os.makedirs(os.path.dirname(rubric_filename), exist_ok=True)
 
     # Load the task spec. This results in a task-specification module,
     # which must have a 'rubric' property that holds a Rubric object.
@@ -130,6 +130,7 @@ def generate_rubric(task_info, rubric_filename):
             "Fatal error: task specification has no 'rubric' attribute,"
             "or 'rubric' value is not a Rubric."
         )
+        sys.exit(1)
 
     # Given a Rubric, evaluate that Rubric in blank mode to produce a
     # report for a rubric HTML file.
@@ -142,6 +143,116 @@ def generate_rubric(task_info, rubric_filename):
     logging.log("...done creating rubric.")
 
 
+def test_specification(task_info, examples_dir):
+    """
+    Tests the specification for a single task, based on the task info
+    and a directory where test submissions should be found.
+
+    Writes log messages using the logging sub-module, so redirect those
+    beforehand if you wish.
+
+    Exits with exit code 1 if any specification tests fail.
+    """
+    logging.log(
+        f"Testing specification:\n"
+        f"    task: {task_info['id']}"
+        f"    examples_dir: {examples_dir}"
+    )
+
+    # Load the task spec. This results in a task-specification module,
+    # which hopefully establishes expectations that accumulate in
+    # `potluck.meta.EXPECTATIONS`.
+    contexts.AutoContext.reset(task_info["target"])
+    spec = load.load_task_spec(task_info)
+    # Attach the loaded spec back into the task info
+    task_info["specification"] = spec
+    if (
+        not hasattr(spec, "rubric")
+     or not isinstance(spec.rubric, rubrics.Rubric)
+    ):
+        logging.log(
+            "Fatal error: task specification has no 'rubric' attribute,"
+            "or 'rubric' value is not a Rubric."
+        )
+        sys.exit(1)
+
+    any_failed = False
+
+    by_user = meta.get_expectations(spec)
+    if len(by_user) == 0:
+        logging.log(
+            "No explicit expectations; only testing solution code."
+        )
+    else:
+        total = sum(len(exp) for exp in by_user.values())
+        logging.log(
+            f"Found {total} expectation(s) for {len(by_user)} example"
+            f" submission(s)."
+        )
+        failed = []
+        for username in by_user:
+            # Resolve which file we're targeting
+            user_folder = os.path.join(
+                examples_dir,
+                username
+            )
+            task_folder = os.path.join(user_folder, task_info["id"])
+            submission_target = os.path.join(
+                task_folder,
+                task_info["target"]
+            )
+
+            # Retrieve expectations list
+            expectations = by_user[username]
+            logging.log(f"Checking expectations for '{username}'...")
+            # Evaluate our rubric against the example submission
+            report = spec.rubric.evaluate(
+                task_info,
+                username,
+                submission_target
+            )
+            # Check the resulting report
+            passed, expl = meta.check_entire_report(report, expectations)
+            # Log the resulting explanation
+            logging.log(expl)
+            status = "passed"
+            if not passed:
+                status = "FAILED"
+                failed.append(username)
+                any_failed = True
+            logging.log(f"...done checking '{username}' ({status}).")
+        if len(failed) > 0:
+            logging.log(f"{len(failed)}/{len(by_user)} examples failed.")
+        else:
+            logging.log("All examples met expectations.")
+
+    logging.log("Checking solution code...")
+    soln_file = os.path.join(
+        os.path.dirname(spec.__file__),
+        "soln",
+        task_info["target"]
+    )
+    soln_report = spec.rubric.evaluate(
+        task_info,
+        "__soln__",
+        soln_file
+    )
+    # Check just defaults for solution report
+    passed, expl = meta.check_entire_report(soln_report, [])
+    status = "passed"
+    if not passed:
+        status = "FAILED"
+        any_failed = True
+    logging.log(expl)
+    logging.log(f"Check of solution code {status}.")
+
+    logging.log("...done checking expectations.")
+
+    if any_failed:
+        logging.log("Exiting due to failed expectations.")
+        sys.exit(1)
+
+
 def evaluate_submission(
     task_info,
     username,
@@ -192,6 +303,7 @@ def evaluate_submission(
             "Fatal error: task specification has no 'rubric' attribute,"
             "or 'rubric' value is not a Rubric."
         )
+        sys.exit(1)
 
     # Given a submission to evaluate and a Rubric, evaluate that Rubric
     # in the context of the submission. This produces a report with
@@ -234,13 +346,13 @@ def setup(config, specs_dir=None, templates_dir=None, resources_dir=None):
 
     # Set up reports system based on templates and resources directories
     # from config
-    if templates_dir is None:
+    if templates_dir is None and config.TEMPLATES_DIRECTORY is not None:
         templates_dir = os.path.join(
             file_utils.potluck_src_dir(),
             config.TEMPLATES_DIRECTORY
         )
 
-    if resources_dir is None:
+    if resources_dir is None and config.RESOURCES_DIRECTORY is not None:
         resources_dir = os.path.join(
             file_utils.potluck_src_dir(),
             config.RESOURCES_DIRECTORY
@@ -260,7 +372,7 @@ def launch_rubric_generation(
     """
     # Ensure logging directory exists if we're logging to a file
     if log_file is not None:
-        os.makedirs(os.path.split(log_file)[0], exist_ok=True)
+        os.makedirs(os.path.dirname(log_file), exist_ok=True)
 
     # Note: all other actions will happen with log file open
     with open(log_file, 'w') if log_file else sys.stdout as lout:
@@ -314,6 +426,71 @@ def launch_rubric_generation(
                 logging.log(report.ERROR_MSG)
 
 
+def launch_specifications_test(
+    config,
+    taskid,
+    log_file=None
+):
+    """
+    Loads a specification and checks any `potluck.meta.Expecation`s
+    defined there. Note that corresponding test submissions must already
+    be present in the evaluation directory.
+
+    Test results are written to the log, which by default is simply
+    printed to stdout; no files are produced (unless logging is directed
+    to a file).
+    """
+    # Ensure logging directory exists if we're logging to a file
+    if log_file is not None:
+        os.makedirs(os.path.dirname(log_file), exist_ok=True)
+
+    # Note: all other actions will happen with log file open
+    with (open(log_file, 'w') if log_file else sys.stdout) as lout:
+        logging.set_log_target(lout)
+
+        # From here on out, we want to log either a completion message or
+        # an error message at the end no matter what
+        done = False
+        try:
+            logging.log(f"This is potluck version {__version__}")
+            logging.log(f"Testing specification for {taskid}...")
+
+            # Load task meta-data
+            task_info_file = os.path.join(
+                config.BASE_DIR,
+                config.TASKS_FILENAME
+            )
+            with open(task_info_file, 'r') as fin:
+                tasks_data = json.load(fin)
+
+            if taskid not in tasks_data["tasks"]:
+                logging.log(
+                    f"Fatal error: Task '{taskid}' does not exist in"
+                    f" task info file '{task_info_file}'"
+                )
+                sys.exit(1)
+
+            task_info = tasks_data["tasks"][taskid]
+            task_info["id"] = taskid
+
+            # Use config to determine directory where submissions live
+            examples_dir = os.path.join(
+                config.BASE_DIR,
+                config.EXAMPLES_DIR
+            )
+
+            test_specification(task_info, examples_dir)
+            logging.log("...specification test complete.")
+            done = True
+            # Now we're done
+
+        finally: # log our completion or error message
+            if done:
+                logging.log(report.DONE_MSG)
+            else:
+                logging.log(report.ERROR_MSG)
+
+
 def launch_evaluation(
     config,
     taskid,
@@ -331,7 +508,7 @@ def launch_evaluation(
 
     # Ensure logging directory exists if we're logging to a file
     if log_file is not None:
-        os.makedirs(os.path.split(log_file)[0], exist_ok=True)
+        os.makedirs(os.path.dirname(log_file), exist_ok=True)
 
     # Note: all other actions will happen with log file open
     with open(log_file, 'w') if log_file else sys.stdout as lout:
