@@ -2,6 +2,8 @@
 Mast: Matcher of ASTs for Python
 Copyright (c) 2017-2018, Benjamin P. Wood, Wellesley College
 
+mast.py
+
 This version is included in potluck instead of codder.
 
 See NOTE below for how to run this file as a script.
@@ -159,8 +161,7 @@ LIT_TYPES = {
 
 TYPED_LIT_VAR_SPEC = (
     (1, 2),
-    re.compile(r'\A_([a-zA-Z0-9]+)_(' + '|'.join(LIT_TYPES.keys()) + ')_\Z')\
-# noqa W605
+    re.compile(r'\A_([a-zA-Z0-9]+)_(' + '|'.join(LIT_TYPES.keys()) + r')_\Z')
 )
 """Spec for names/types of typed literal pattern variables."""
 
